#include "mbed.h"

#include "MARMEX_OB_oled.h"
#include "ov7670.h"

DigitalOut myled(P0_22);
MARMEX_OB_oled   oled1( p5, p7,  p8, p30, p11 ); // mosi, sclk, cs, rst, power_control

#define SIZEX (160)
#define SIZEY (120)

void test_oled() {
    oled1.background( 0x000000 );
    oled1.cls();

    int colorbar_width  = MARMEX_OB_oled::WIDTH / 8;
    for ( int i = 0; i < 8; i++ )
        oled1.fill( colorbar_width * i, 0, colorbar_width, MARMEX_OB_oled::HEIGHT, ((i & 0x4) ? 0xFF0000 : 0x000000) | ((i & 0x2) ? 0x00FF00 : 0x000000) | ((i & 0x1) ? 0x0000FF : 0x000000) );

    oled1.fill(  50,  50,  64,  64, 0xCCCCCC );;

    oled1.locate( 0, 3 );
    oled1.printf( "Hello World!" );
    oled1.locate( 0, 4 );
    oled1.printf( "SPI = %s", MERMEX_OB_SPI_MODE_STR );

    for (int i = 0; i < MARMEX_OB_oled::WIDTH; i++ ) {
        oled1.pixel( i, 80 + sin( (float)i / 5.0 ) * 10, 0x000000 );
    }
}

int main() {
	// OLEDのテスト表示
	test_oled();
	oled1.locate( 0, 3 );

	// カメラの初期化
    OV7670 camera(
        p28,p27,       // SDA,SCL(I2C / SCCB)
        p21,p22,p20,   // VSYNC,HREF,WEN(FIFO)
        p19,p18,p17,p16,p15,p14,p13,p12, // D7-D0
        p23,p24,p25) ; // RRST,OE,RCLK
    // カメラのリセット
    oled1.printf("Camera resetting..¥r¥n") ;
    camera.Reset() ;
	// カメラの初期化
    oled1.printf("Camera initializing..¥r¥n") ;
    camera.InitQQVGA() ;
    // カメラの画像を取り込む
    while(1) {
    	myled = 1;
    	camera.CaptureNext() ;
    	while(camera.CaptureDone() == false) ;
    	camera.ReadStart() ;
    	myled = 0;
    	for (int y = 0; y < SIZEY; y++) {
    		int r, g, b,d1, d2;
    		for (int x = 0; x < SIZEX; x++) {
    			d1 = camera.ReadOneByte() ; // upper nibble is XXX , lower nibble is B
    			d2 = camera.ReadOneByte() ; // upper nibble is G   , lower nibble is R
				b = (d1 & 0x0F) ;
				g = (d2 & 0xF0) >> 4 ;
				r = (d2 & 0x0F) ;
				if (y < 120 && x < 128)
					oled1.pixel( x, y, (r << 20) | (g << 12) | (b << 4));
    		}
    	}
    }
}
